@@ -1,6 +1,6 @@
 #!/bin/bash
 
-MY_VERSION="1.12"
+MY_VERSION="1.14c"
 
 # ------------------------------------------------------------------------------------------
 #                         -= Arno's Iptables Firewall(AIF) =-
@@ -8,7 +8,7 @@ MY_VERSION="1.12"
 #
 #                           ~ In memory of my dear father ~
 #
-# (C) Copyright 2001-2019 by Arno van Amersfoort
+# (C) Copyright 2001-2021 by Arno van Amersfoort
 # Homepage              : https://rocky.eld.leidenuniv.nl/
 # Email                 : a r n o v a AT r o c k y DOT e l d DOT l e i d e n u n i v DOT n l
 #                         (note: you must remove all spaces and substitute the @ and the .
@@ -86,10 +86,7 @@ shell_diff()
 
   if [ "$FILE1_DATA" != "$FILE2_DATA" ]; then
     # If mismatch, check whether it's only the comments that differ
-    FILE1_DATA="$(echo "$FILE1_DATA" |sed 's/#.*//')"
-    FILE2_DATA="$(echo "$FILE2_DATA" |sed 's/#.*//')"
-
-    if [ "$FILE1_DATA" = "$FILE2_DATA" ]; then
+    if [ "${FILE1_DATA%\#*}" = "${FILE2_DATA%\#*}" ]; then
       return 1 # Only comments differ
     fi
 
@@ -102,68 +99,71 @@ shell_diff()
 
 copy_ask_if_exist()
 {
-  local diff_retval=-1
-  local retval
+  local DIFF_RETVAl=-1
+  local RETVAL
+  local DEFAULT_YN="${3:-'n'}" # Default to n(o)
+  local BACKUP_EXT="$4"
+  local FALLBACK_EXT="$5"
 
   if [ -z "$(find "$1" -type f)" ]; then
-    echo "ERROR: Missing source file(s) \"$1\""
+    echo "ERROR: Missing source file(s) \"$1\"" >&2
     exit 2
   fi
 
   unset IFS
-  for source in `find "$1" -type f |grep -v -e '/\.svn/' -e '/\.git/'`; do
+  for SOURCE in `find "$1" -type f |grep -v -e '/\.svn/' -e '/\.git/'`; do
     if echo "$2" |grep -q '/$'; then
-      fn="$(echo "$source" |sed "s,^$1,,")"
-      if [ -z "$fn" ]; then
-        target="${2}$(basename "$1")"
+      FN="${SOURCE#$1}"
+      if [ -z "$FN" ]; then
+        TARGET="${2}$(basename "$1")"
       else
-        target="${2}${fn}"
+        TARGET="${2}${FN}"
       fi
-      target_dir="$2"
+      TARGET_DIR="$2"
     else
-      target="$2"
-      target_dir="$(dirname "$2")"
+      TARGET="$2"
+      TARGET_DIR="$(dirname "$2")"
     fi
 
-    if [ ! -d "$target_dir" ]; then
-      printf "\033[40m\033[1;31m* WARNING: Target directory $target_dir does not exist. Skipping copy of $source!\033[0m\n" >&2
+    if [ ! -d "$TARGET_DIR" ]; then
+      printf "\033[40m\033[1;31m* WARNING: Target directory $TARGET_DIR does not exist. Skipping copy of $SOURCE!\033[0m\n" >&2
       continue
     fi
 
-    if [ -f "$source" -a -f "$target" ]; then
+    if [ -f "$SOURCE" -a -f "$TARGET" ]; then
       # Ignore files that are the same in the target
-      shell_diff "$source" "$target"
-      diff_retval=$? # 0 = full match, 1 = match (excluding comments), 2 = full mismatch (including comments)
+      shell_diff "$SOURCE" "$TARGET"
+      DIFF_RETVAL=$? # 0 = full match, 1 = match (excluding comments), 2 = full mismatch (including comments)
 
-      if [ $diff_retval -eq 2 ] && ! get_user_yn "File \"$target\" already exists. Overwrite" "n"; then
-        if [ -z "$3" ]; then
+      if [ $DIFF_RETVAL -eq 2 ] && ! get_user_yn "File \"$TARGET\" already exists. Overwrite" "$DEFAULT_YN"; then
+        if [ -z "$FALLBACK_EXT" ]; then
           echo "Skipped..."
           continue
         else
           # Copy as e.g. .dist-file:
-          target="${target}.${3}"
-          rm -f "$target"
+          TARGET="${TARGET}.${FALLBACK_EXT}"
+          rm -f "$TARGET"
         fi
       fi
     fi
 
-    retval=0
-    if [ $diff_retval -eq 2 ]; then
+    RETVAL=0
+    if [ $DIFF_RETVAL -eq 2 -a -n "$BACKUP_EXT" ]; then
       # copy file & create backup of old file if exists
-      cp -bv --preserve=mode,timestamps "$source" "$target"
-      retval=$?
+      cp -v --suffix=".${BACKUP_EXT}" --backup=simple --preserve=mode,timestamps "$SOURCE" "$TARGET"
+      RETVAL=$?
     else
       # Only comments mismatch, so no point in keeping a backup file
-      cp -v --preserve=mode,timestamps "$source" "$target"
-      retval=$?
+      cp -v --preserve=mode,timestamps "$SOURCE" "$TARGET"
+      RETVAL=$?
     fi
 
-    if [ $retval -ne 0 ]; then
-      echo "ERROR: Copy of \"$source\" to \"$target\" failed!" >&2
+    if [ $RETVAL -ne 0 ]; then
+      echo "ERROR: Copy of \"$SOURCE\" to \"$TARGET\" failed!" >&2
       exit 3
     fi
 
-    chown 0:0 "$target"
+    chown 0:0 "$TARGET"
   done
 
   return 0
@@ -178,43 +178,43 @@ copy_skip_if_exist()
   fi
 
   unset IFS
-  for source in `find "$1" -type f |grep -v -e '/\.svn/' -e '/\.git/'`; do
+  for SOURCE in `find "$1" -type f |grep -v -e '/\.svn/' -e '/\.git/'`; do
     if echo "$2" |grep -q '/$'; then
-      fn="$(echo "$source" |sed "s,^$1,,")"
-      if [ -z "$fn" ]; then
-        target="$2$(basename "$1")"
+      FN="${SOURCE#$1}"
+      if [ -z "$FN" ]; then
+        TARGET="$2$(basename "$1")"
       else
-        target="$2$fn"
+        TARGET="$2$FN"
       fi
-      target_dir="$2"
+      TARGET_DIR="$2"
     else
-      target="$2"
-      target_dir="$(dirname "$2")"
+      TARGET="$2"
+      TARGET_DIR="$(dirname "$2")"
     fi
 
-    if [ ! -d "$target_dir" ]; then
-      printf "\033[40m\033[1;31m* WARNING: Target directory $target_dir does not exist. Skipping copy of $source!\033[0m\n" >&2
+    if [ ! -d "$TARGET_DIR" ]; then
+      printf "\033[40m\033[1;31m* WARNING: Target directory $TARGET_DIR does not exist. Skipping copy of $SOURCE!\033[0m\n" >&2
       continue
     fi
 
-    if [ -f "$target" ]; then
+    if [ -f "$TARGET" ]; then
       if [ -z "$3" ]; then
-        echo "* File \"$target\" already exists. Skipping copy of $source"
+        echo "* File \"$TARGET\" already exists. Skipping copy of $SOURCE"
         continue
       else
         # Copy as e.g. .dist-file:
-        target="${target}.${3}"
-        rm -f "$target"
+        TARGET="${TARGET}.${3}"
+        rm -f "$TARGET"
       fi
     fi
 
     # NOTE: Always copy, even if contents is the same to make sure permissions are updated 
-    if ! cp -v --preserve=mode,timestamps "$source" "$target"; then
-      echo "ERROR: Copy of \"$source\" to \"$target!\" failed!" >&2
+    if ! cp -v --preserve=mode,timestamps "$SOURCE" "$TARGET"; then
+      echo "ERROR: Copy of \"$SOURCE\" to \"$TARGET!\" failed!" >&2
       exit 3
     fi
 
-    chown 0:0 "$target"
+    chown 0:0 "$TARGET"
   done
 
   return 0
@@ -229,32 +229,32 @@ copy_overwrite()
   fi
 
   unset IFS
-  for source in `find "$1" -type f |grep -v -e '/\.svn/' -e '/\.git/'`; do
+  for SOURCE in `find "$1" -type f |grep -v -e '/\.svn/' -e '/\.git/'`; do
     if echo "$2" |grep -q '/$'; then
-      fn="$(echo "$source" |sed "s,^$1,,")"
-      if [ -z "$fn" ]; then
-        target="$2$(basename "$1")"
+      FN="${SOURCE#$1}"
+      if [ -z "$FN" ]; then
+        TARGET="$2$(basename "$1")"
       else
-        target="$2$fn"
+        TARGET="$2$FN"
       fi
-      target_dir="$2"
+      TARGET_DIR="$2"
     else
-      target="$2"
-      target_dir="$(dirname "$2")"
+      TARGET="$2"
+      TARGET_DIR="$(dirname "$2")"
     fi
 
-    if [ ! -d "$target_dir" ]; then
-      printf "\033[40m\033[1;31m* WARNING: Target directory $target_dir does not exist. Skipping copy of $source!\033[0m\n" >&2
+    if [ ! -d "$TARGET_DIR" ]; then
+      printf "\033[40m\033[1;31m* WARNING: Target directory $TARGET_DIR does not exist. Skipping copy of $SOURCE!\033[0m\n" >&2
       continue
     fi
 
     # NOTE: Always copy, even if contents is the same to make sure permissions are updated
-    if ! cp -fv --preserve=mode,timestamps "$source" "$target"; then
-      echo "ERROR: Copy of \"$source\" to \"$target\" failed!" >&2
+    if ! cp -fv --preserve=mode,timestamps "$SOURCE" "$TARGET"; then
+      echo "ERROR: Copy of \"$SOURCE\" to \"$TARGET\" failed!" >&2
       exit 3
     fi
 
-    chown 0:0 "$target"
+    chown 0:0 "$TARGET"
   done
 
   return 0
@@ -304,6 +304,18 @@ check_18_version()
 }
 
 
+check_dist_version()
+{
+  if [ -f /usr/sbin/arno-iptables-firewall ]; then
+    if ! get_user_yn "WARNING: It seems a distribution version is already installed. It's *STRONGLY* recommended to remove it first. Continue anyway" "n"; then
+      return 1
+    fi
+  fi
+
+  return 0
+}
+
+
 # Check plugins for (old) versions with different priority
 check_plugins()
 {
@@ -321,6 +333,89 @@ check_plugins()
 }
 
 
+setup_start_scripts()
+{
+  # Install init.d script, but only if init.d folder exists
+  if [ -d "/etc/init.d" ]; then
+    copy_overwrite ./etc/init.d/arno-iptables-firewall /etc/init.d/
+  fi
+
+  # Make sure only one service file exists in /lib/.. or /usr/lib/ where we prefer /lib/
+  rm -f /usr/lib/systemd/system/arno-iptables-firewall.service
+
+  # Install service file if systemd directory is available, use fallbacks to support different systems
+  if [ -d "/lib/systemd/system" ]; then
+    copy_overwrite ./lib/systemd/system/arno-iptables-firewall.service /lib/systemd/system/
+  elif [ -d "/usr/lib/systemd/system" ]; then
+    copy_overwrite ./lib/systemd/system/arno-iptables-firewall.service /usr/lib/systemd/system/
+  elif [ -d "/etc/systemd/system" ]; then
+    copy_ask_if_exist ./lib/systemd/system/arno-iptables-firewall.service /etc/systemd/system/ "y"
+  else
+    echo "NOTE: Could not find any systemd/system directory, skipping systemd configuration" >&2
+  fi
+
+  RC_PATH="/etc"
+  # Check for Redhat/SUSE rc.d
+  if [ -d "/etc/rc.d" ]; then
+    RC_PATH="/etc/rc.d"
+  fi
+
+  # Remove any symlinks in rc*.d out of the way
+  rm -f $RC_PATH/rc0.d/*arno-iptables-firewall
+  rm -f $RC_PATH/rc1.d/*arno-iptables-firewall
+  rm -f $RC_PATH/rc2.d/*arno-iptables-firewall
+  rm -f $RC_PATH/rc3.d/*arno-iptables-firewall
+  rm -f $RC_PATH/rc4.d/*arno-iptables-firewall
+  rm -f $RC_PATH/rc5.d/*arno-iptables-firewall
+  rm -f $RC_PATH/rc6.d/*arno-iptables-firewall
+  rm -f $RC_PATH/rcS.d/*arno-iptables-firewall
+
+  if get_user_yn "Do you want to start the firewall at boot" "y"; then
+    DONE=0
+
+    if check_command systemctl; then
+      if systemctl enable arno-iptables-firewall; then
+        echo "* Successfully enabled service with systemctl"
+        DONE=1
+      fi
+    elif check_command update-rc.d; then
+      # Note: Currently update-rc.d doesn't seem to properly use the init script's LSB header, so specify explicitly
+      if update-rc.d -f arno-iptables-firewall start 11 S . stop 10 0 6 .; then
+        echo "* Successfully enabled service with update-rc.d"
+        DONE=1
+      fi
+    elif check_command chkconfig; then
+      if chkconfig --add arno-iptables-firewall && chkconfig arno-iptables-firewall on; then
+        echo "* Successfully enabled service with chkconfig"
+        DONE=1
+      fi
+    else
+      if [ -d "$RC_PATH/rcS.d" ]; then
+        if ln -sv /etc/init.d/arno-iptables-firewall "$RC_PATH/rcS.d/S11arno-iptables-firewall" &&
+          ln -sv /etc/init.d/arno-iptables-firewall "$RC_PATH/rc0.d/K10arno-iptables-firewall" &&
+          ln -sv /etc/init.d/arno-iptables-firewall "$RC_PATH/rc6.d/K10arno-iptables-firewall"; then
+          echo "* Successfully enabled service through $RC_PATH/rcS.d/ symlink"
+          DONE=1
+        fi
+      elif [ -d "$RC_PATH/rc2.d" ]; then
+        if ln -sv /etc/init.d/arno-iptables-firewall "$RC_PATH/rc2.d/S09arno-iptables-firewall" &&
+          ln -sv /etc/init.d/arno-iptables-firewall "$RC_PATH/rc0.d/K91arno-iptables-firewall" &&
+          ln -sv /etc/init.d/arno-iptables-firewall "$RC_PATH/rc6.d/K91arno-iptables-firewall"; then
+          echo "* Successfully enabled service through $RC_PATH/rc2.d/ symlink"
+          DONE=1
+        fi
+      else
+        echo "WARNING: Unable to detect /rc2.d or /rcS.d directories. Skipping runlevel symlinks" >&2
+      fi
+    fi
+
+    if [ $DONE -eq 0 ]; then
+      echo "ERROR: Unable to setup automatic start at boot. Please investigate" >&2
+    fi
+  fi
+}
+
+
 # main line:
 AIF_VERSION="$(grep "MY_VERSION=" ./bin/arno-iptables-firewall |sed -e "s/^MY_VERSION=\"//" -e "s/\"$//")"
 
@@ -331,16 +426,22 @@ echo "--------------------------------------------------------------------------
 sanity_check
 
 # We want to run in the dir the install script is in
-cd "$(dirname $0)"
+cd "$(dirname $0)" || exit 1
 
 if ! get_user_yn "Continue install" "n"; then
   echo "*Install aborted"
   exit 1
 fi
 
-# Make sure there still isn't an old version installed
+# Make sure an old version is not still installed
 check_18_version
 
+# Make sure a dist version is not already installed
+if ! check_dist_version; then
+  echo "*Install aborted"
+  exit 1
+fi
+
 copy_overwrite ./bin/arno-iptables-firewall /usr/local/sbin/
 copy_overwrite ./bin/arno-fwfilter /usr/local/bin/
 
@@ -350,7 +451,7 @@ rm -f /usr/local/sbin/arno-fwfilter
 mkdir -pv /usr/local/share/arno-iptables-firewall/plugins || exit 1
 copy_overwrite ./share/arno-iptables-firewall/ /usr/local/share/arno-iptables-firewall/
 
-if [ ! -e /usr/local/sbin/traffic-accounting-show ]; then 
+if [ ! -f /usr/local/sbin/traffic-accounting-show ]; then 
   ln -sv /usr/local/share/arno-iptables-firewall/plugins/traffic-accounting-show /usr/local/sbin/traffic-accounting-show
 fi
 
@@ -362,45 +463,32 @@ gzip -c -v ./share/man/man1/arno-fwfilter.1 >/usr/local/share/man/man8/arno-fwfi
 mkdir -pv /usr/local/share/doc/arno-iptables-firewall || exit 1
 copy_overwrite ./README /usr/local/share/doc/arno-iptables-firewall/
 
-# Install init.d script, but only if init.d folder exists
-if [ -d "/etc/init.d" ]; then
-  copy_ask_if_exist ./etc/init.d/arno-iptables-firewall /etc/init.d/
-fi
-
-# Make sure only one service file exists in /lib/.. or /usr/lib/ where we prefer /lib/
-rm -f /usr/lib/systemd/system/arno-iptables-firewall.service
-
-# Install service file if systemd directory is available, use fallbacks to support different systems
-if [ -d "/lib/systemd/system" ]; then
-  copy_overwrite ./lib/systemd/system/arno-iptables-firewall.service /lib/systemd/system/
-elif [ -d "/usr/lib/systemd/system" ]; then
-  copy_overwrite ./lib/systemd/system/arno-iptables-firewall.service /usr/lib/systemd/system/
-elif [ -d "/etc/systemd/system" ]; then
- copy_ask_if_exist ./lib/systemd/system/arno-iptables-firewall.service /etc/systemd/system/
-else
-  echo "NOTE: Could not find any systemd/system directory, skipping systemd configuration" >&2
-fi
-
 # Install rsyslog config file (if rsyslog is available)
 if [ -d "/etc/rsyslog.d" ]; then
-  copy_ask_if_exist ./etc/rsyslog.d/arno-iptables-firewall.conf /etc/rsyslog.d/
+  copy_ask_if_exist ./etc/rsyslog.d/arno-iptables-firewall.conf /etc/rsyslog.d/ "y" "old" "dist"
 fi
 
+copy_ask_if_exist ./etc/logrotate.d/arno-iptables-firewall /etc/logrotate.d/ "y"
+
 mkdir -pv /etc/arno-iptables-firewall || exit 1
 
 copy_overwrite ./etc/arno-iptables-firewall/firewall.conf /etc/arno-iptables-firewall/firewall.conf.dist
-copy_ask_if_exist ./etc/arno-iptables-firewall/firewall.conf /etc/arno-iptables-firewall/ 
+copy_ask_if_exist ./etc/arno-iptables-firewall/firewall.conf /etc/arno-iptables-firewall/
 
 copy_skip_if_exist ./etc/arno-iptables-firewall/custom-rules /etc/arno-iptables-firewall/
 
 mkdir -pv /etc/arno-iptables-firewall/plugins || exit 1
-copy_ask_if_exist ./etc/arno-iptables-firewall/plugins/ /etc/arno-iptables-firewall/plugins/ "dist"
+copy_ask_if_exist ./etc/arno-iptables-firewall/plugins/ /etc/arno-iptables-firewall/plugins/ "n" "old" "dist"
 
 mkdir -pv /etc/arno-iptables-firewall/conf.d || exit 1
 echo "Files with a .conf extension in this directory will be sourced by the environment file" >/etc/arno-iptables-firewall/conf.d/README
 
+# Check old plugins
 check_plugins
 
+
+setup_start_scripts
+
 echo ""
 echo "** Install done **"
 echo ""
